@@ -14,6 +14,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import json
+import os
+import shlex
+import traceback
+
+import docker
+
+from ansible.module_utils.basic import AnsibleModule
+
 DOCUMENTATION = '''
 ---
 module: kolla_docker
@@ -40,6 +49,7 @@ options:
       - get_container_state
       - pull_image
       - remove_container
+      - remove_image
       - remove_volume
       - recreate_or_restart_container
       - restart_container
@@ -183,12 +193,6 @@ options:
     required: False
     default: False
     type: bool
-  client_timeout:
-    description:
-      - Docker client timeout in seconds
-    required: False
-    default: 120
-    type: int
 author: Sam Yaple
 '''
 
@@ -216,15 +220,12 @@ EXAMPLES = '''
       kolla_docker:
         action: remove_volume
         name: name_of_volume
+    - name: Remove image
+      kolla_docker:
+        action: remove_image
+        image: name_of_image
 '''
 
-import json
-import os
-import shlex
-import traceback
-
-import docker
-
 
 def get_docker_client():
     return docker.APIClient
@@ -243,8 +244,7 @@ class DockerWorker(object):
         # tls_config = self.generate_tls()
 
         options = {
-            'version': self.params.get('api_version'),
-            'timeout': self.params.get('client_timeout'),
+            'version': self.params.get('api_version')
         }
 
         self.dc = get_docker_client()(**options)
@@ -474,7 +474,8 @@ class DockerWorker(object):
             'memswap_limit': 'MemorySwap', 'cpu_period': 'CpuPeriod',
             'cpu_quota': 'CpuQuota', 'cpu_shares': 'CpuShares',
             'cpuset_cpus': 'CpusetCpus', 'cpuset_mems': 'CpusetMems',
-            'kernel_memory': 'KernelMemory', 'blkio_weight': 'BlkioWeight'}
+            'kernel_memory': 'KernelMemory', 'blkio_weight': 'BlkioWeight',
+            'ulimits': 'Ulimits'}
         unsupported = set(new_dimensions.keys()) - \
             set(dimension_map.keys())
         if unsupported:
@@ -486,13 +487,29 @@ class DockerWorker(object):
             # NOTE(mgoddard): If a resource has been explicitly requested,
             # check for a match. Otherwise, ensure is is set to the default.
             if key1 in new_dimensions:
-                if new_dimensions[key1] != current_dimensions[key2]:
+                if key1 == 'ulimits':
+                    if self.compare_ulimits(new_dimensions[key1],
+                                            current_dimensions[key2]):
+                        return True
+                elif new_dimensions[key1] != current_dimensions[key2]:
                     return True
             elif current_dimensions[key2]:
                 # The default values of all currently supported resources are
                 # '' or 0 - both falsey.
                 return True
 
+    def compare_ulimits(self, new_ulimits, current_ulimits):
+        # The new_ulimits is dict, we need make it to a list of Ulimit
+        # instance.
+        new_ulimits = self.build_ulimits(new_ulimits)
+
+        def key(ulimit):
+            return ulimit['Name']
+
+        if current_ulimits is None:
+            current_ulimits = []
+        return sorted(new_ulimits, key=key) != sorted(current_ulimits, key=key)
+
     def compare_command(self, container_info):
         new_command = self.params.get('command')
         if new_command is not None:
@@ -606,6 +623,40 @@ class DockerWorker(object):
 
         return vol_list, vol_dict
 
+    def parse_dimensions(self, dimensions):
+        # When the data object contains types such as
+        # docker.types.Ulimit, Ansible will fail when these are
+        # returned via exit_json or fail_json. HostConfig is derived from dict,
+        # but its constructor requires additional arguments.
+        # to avoid that, here do copy the dimensions and return a new one.
+        dimensions = dimensions.copy()
+
+        supported = {'cpu_period', 'cpu_quota', 'cpu_shares',
+                     'cpuset_cpus', 'cpuset_mems', 'mem_limit',
+                     'mem_reservation', 'memswap_limit',
+                     'kernel_memory', 'blkio_weight', 'ulimits'}
+        unsupported = set(dimensions) - supported
+        if unsupported:
+            self.module.exit_json(failed=True,
+                                  msg=repr("Unsupported dimensions"),
+                                  unsupported_dimensions=unsupported)
+
+        ulimits = dimensions.get('ulimits')
+        if ulimits:
+            dimensions['ulimits'] = self.build_ulimits(ulimits)
+
+        return dimensions
+
+    def build_ulimits(self, ulimits):
+        ulimits_opt = []
+        for key, value in ulimits.items():
+            soft = value.get('soft')
+            hard = value.get('hard')
+            ulimits_opt.append(docker.types.Ulimit(name=key,
+                                                   soft=soft,
+                                                   hard=hard))
+        return ulimits_opt
+
     def build_host_config(self, binds):
         options = {
             'network_mode': 'host',
@@ -617,17 +668,11 @@ class DockerWorker(object):
             'volumes_from': self.params.get('volumes_from')
         }
 
-        if self.params.get('dimensions'):
-            supported = {'cpu_period', 'cpu_quota', 'cpu_shares',
-                         'cpuset_cpus', 'cpuset_mems', 'mem_limit',
-                         'mem_reservation', 'memswap_limit',
-                         'kernel_memory', 'blkio_weight'}
-            unsupported = set(self.params.get('dimensions')) - supported
-            if unsupported:
-                self.module.exit_json(failed=True,
-                                      msg=repr("Unsupported dimensions"),
-                                      unsupported_dimensions=unsupported)
-            options.update(self.params.get('dimensions'))
+        dimensions = self.params.get('dimensions')
+
+        if dimensions:
+            dimensions = self.parse_dimensions(dimensions)
+            options.update(dimensions)
 
         if self.params.get('restart_policy') in ['on-failure',
                                                  'always',
@@ -688,11 +733,6 @@ class DockerWorker(object):
         # If config_strategy is COPY_ONCE or container's parameters are
         # changed, try to start a new one.
         if config_strategy == 'COPY_ONCE' or self.check_container_differs():
-            # NOTE(mgoddard): Pull the image if necessary before stopping the
-            # container, otherwise a failure to pull the image will leave the
-            # container stopped.
-            if not self.check_image():
-                self.pull_image()
             self.stop_container()
             self.remove_container()
             self.start_container()
@@ -812,6 +852,26 @@ class DockerWorker(object):
                     )
                 raise
 
+    def remove_image(self):
+        if self.check_image():
+            self.changed = True
+            try:
+                self.dc.remove_image(image=self.params.get('image'))
+            except docker.errors.APIError as e:
+                if e.response.status_code == 409:
+                    self.module.fail_json(
+                        failed=True,
+                        msg="Image '{}' is currently in-use".format(
+                            self.params.get('image')
+                        )
+                    )
+                elif e.response.status_code == 500:
+                    self.module.fail_json(
+                        failed=True,
+                        msg="Server error"
+                    )
+                raise
+
 
 def generate_module():
     # NOTE(jeffrey4l): add empty string '' to choices let us use
@@ -823,9 +883,9 @@ def generate_module():
                              'create_volume', 'get_container_env',
                              'get_container_state', 'pull_image',
                              'recreate_or_restart_container',
-                             'remove_container', 'remove_volume',
-                             'restart_container', 'start_container',
-                             'stop_container']),
+                             'remove_container', 'remove_image',
+                             'remove_volume', 'restart_container',
+                             'start_container', 'stop_container']),
         api_version=dict(required=False, type='str', default='auto'),
         auth_email=dict(required=False, type='str'),
         auth_password=dict(required=False, type='str', no_log=True),
@@ -866,7 +926,6 @@ def generate_module():
         volumes_from=dict(required=False, type='list'),
         dimensions=dict(required=False, type='dict', default=dict()),
         tty=dict(required=False, type='bool', default=False),
-        client_timeout=dict(required=False, type='int', default=120),
     )
     required_if = [
         ['action', 'pull_image', ['image']],
@@ -878,6 +937,7 @@ def generate_module():
         ['action', 'get_container_state', ['name']],
         ['action', 'recreate_or_restart_container', ['name']],
         ['action', 'remove_container', ['name']],
+        ['action', 'remove_image', ['image']],
         ['action', 'remove_volume', ['name']],
         ['action', 'restart_container', ['name']],
         ['action', 'stop_container', ['name']]
@@ -926,7 +986,6 @@ def main():
         module.fail_json(changed=True, msg=repr(traceback.format_exc()),
                          **getattr(dw, 'result', {}))
 
-# import module snippets
-from ansible.module_utils.basic import *  # noqa
+
 if __name__ == '__main__':
     main()
