@@ -16,23 +16,26 @@ import argparse
 import yaml
 
 
+def mergepwd(old, new, final):
+    with open(old, "r") as old_file:
+        old_passwords = yaml.safe_load(old_file)
+
+    with open(new, "r") as new_file:
+        new_passwords = yaml.safe_load(new_file)
+
+    new_passwords.update(old_passwords)
+
+    with open(final, "w") as destination:
+        yaml.safe_dump(new_passwords, destination, default_flow_style=False)
+
+
 def main():
     parser = argparse.ArgumentParser()
     parser.add_argument("--old", help="old password file", required=True)
     parser.add_argument("--new", help="new password file", required=True)
     parser.add_argument("--final", help="merged password file", required=True)
     args = parser.parse_args()
-
-    with open(args.old, "r") as old_file:
-        old_passwords = yaml.safe_load(old_file)
-
-    with open(args.new, "r") as new_file:
-        new_passwords = yaml.safe_load(new_file)
-
-    new_passwords.update(old_passwords)
-
-    with open(args.final, "w") as destination:
-        yaml.safe_dump(new_passwords, destination, default_flow_style=False)
+    mergepwd(args.old, args.new, args.final)
 
 
 if __name__ == '__main__':
